import https from 'https';
import Http from '../Http';

/**
 * Performs HTTP requests through the browser's AJAX features.
 */
export default class NodeHttp extends Http {
    /**
     * Creates a new NodeHttp instance.
     *
     * @param base the base URL to perform all requests against
     * @param authProvider the authentication provider to get authentication tokens from
     */
    constructor(base, authProvider) {
        super();
        this.base = base;
        this.authProvider = authProvider;
    }

    /**
     * Performs the request.
     *
     * @param method the request method
     * @param url the request URL
     * @param body the request body, if any
     * @param resolve the callback to call on success
     * @param reject the callback to call on failure
     * @param token if present, the authorization token
     * @param bodyType the datatype of the body
     *
     * @private
     */
    _performRequest(method, url, body, resolve, reject, token, bodyType) {
        const headers = (bodyType === Http.MULTIPART_MIME) ? {} : { 'Content-Type': bodyType };

        if (token) {
            headers['Auth-Token'] = token.token;
            headers['Authorization'] = `AutaToken ${token.token}`;
            headers['Accept'] = Http.JSON_MIME;
        }

        const req = https.request(new URL(url, this.base), {
            method: method,
            headers: headers
        });

        const resContainer  = {str: ''};

        req.on('response', res => {
            res.on('data', data => resContainer.str += data);
            res.on('end', () => {
                const data = JSON.parse(resContainer.str);
                const meta = {status: res.statusCode};

                if (res.statusCode === 403) {
                    this.authProvider.handleInvalid().then(() => reject(this.reply(data, meta)));
                    return;
                }

                if (res.statusCode >= 200 && res.statusCode < 400) {
                    resolve(this.reply(data, meta));
                } else {
                    reject(this.reply(data, meta));
                }
            });
        });
        req.on('error', reject);

        req.end((bodyType === Http.JSON_MIME && body) ? JSON.stringify(body) : body, 'utf8');
    }

    /**
     * Performs a request.
     *
     * @param method the HTTP request method
     * @param url the URL to perform the request against
     * @param body the request body, if any
     * @param unauthenticated if true, no authentication token is attached (useful for /login calls)
     * @param bodyType the datatype of the body
     *
     * @returns {Promise<Object>} a promise containing the response to the request
     */
    request(method, url, body, unauthenticated, bodyType = Http.JSON_MIME) {
        return new Promise((resolve, reject) => {
            if (unauthenticated) {
                this._performRequest(method, url, body, resolve, reject, undefined, bodyType);
            } else {
                const token = this.authProvider.getToken().then(token => {
                    this._performRequest(method, url, body, resolve, reject, token, bodyType);
                }).catch(reject);
            }
        });
    }
}
