import Http from '../Http';
import Token from '../../Token';

/**
 * Performs HTTP requests through the browser's AJAX features.
 */
export default class AjaxHttp extends Http {
    /**
     * Creates a new AjaxHttp instance.
     *
     * @param base the base URL all requests should be performed against
     * @param authProvider the authentication provider to get authentication tokens from
     * @param debug if true, enables detailed logging for all requests
     */
    constructor(base, authProvider, debug) {
        super();
        this.host = base;
        this.authProvider = authProvider;
        this.debug = debug;
    }

    /**
     * Performs the request.
     *
     * @param xhr the XMLHttpRequest context
     * @param method the request method
     * @param url the url the request is against
     * @param body the request body, if any
     * @param resolve the callback to call on success
     * @param reject the callback to call on failure
     * @param hasToken if true, the request contains an authentication token
     * @param bodyType the datatype of the body
     *
     * @private
     */
    _performRequest(xhr, method, url, body, resolve, reject, hasToken, bodyType) {
        xhr.responseType = 'json';
        xhr.onload = () => {
            const meta = {status: xhr.status};

            if (xhr.status === 403 && url !== '/api/v1/user/logout') {
                this.authProvider.setToken(new Token(undefined, undefined)).then(() =>
                    this.authProvider.handleInvalid()
                        .then(() => reject(this.reply(xhr.response, meta)))
                        .catch(reject)
                ).catch(reject);
                return;
            }

            ((xhr.status >= 200 && xhr.status < 400) ? resolve : reject)(this.reply(xhr.response, meta));
        };

        this.log(`${hasToken ? 'AUTH' : 'ANON'} ${method} ${url}${body ? ' +body' : ''}`);

        xhr.send((bodyType === Http.JSON_MIME && body) ? JSON.stringify(body) : body);
    }

    /**
     * Performs a request.
     *
     * @param method the HTTP request method
     * @param url the URL to perform the request against
     * @param body the request body, if any
     * @param unauthenticated if true, no authentication token is attached (useful for /login calls)
     * @param bodyType the datatype of the body
     *
     * @returns {Promise<Object>} a promise containing the response to the request
     */
    request(method, url, body, unauthenticated, bodyType = Http.JSON_MIME) {
        return new Promise((resolve, reject) => {
            const xhr = new XMLHttpRequest();
            xhr.open(method, new URL(url, this.host).toString());
            if (bodyType !== Http.MULTIPART_MIME) {
                xhr.setRequestHeader('Content-Type', bodyType);
            }

            if (unauthenticated) {
                this._performRequest(xhr, method, url, body, resolve, reject, false, bodyType);
            } else {
                this.authProvider.getToken().then(token => {
                    const hasToken = token && token.token;
                    if (hasToken) {
                        xhr.withCredentials = true;
                        xhr.setRequestHeader('Auth-Token', token.token); // Compatibility with non-compliant APIs
                        xhr.setRequestHeader('Authorization', `AutaToken ${token.token}`);
                    }
                    this._performRequest(xhr, method, url, body, resolve, reject, hasToken, bodyType);
                }).catch(reject);
            }
        });
    }

    log(msg) {
        if (!this.debug) return;

        console.log(`XHR DEBUG: ${msg}`);
    }
}
